import re
from typing import Dict, Iterator, Match, NewType, Tuple

from .metadata import NAME, VERSION
from .utils import Code, code_from

Flake8Message = NewType("Flake8Message", str)
Flake8Result = NewType("Flake8Result", Tuple[int, Flake8Message])

WORD_CODES: Dict[str, Code] = {
    "FIXME": code_from("100"),  # noqa: T100
    "TODO": code_from("101"),  # noqa: T101
    "XXX": code_from("102"),  # noqa: T102
    "HACK": code_from("103"),  # noqa: T103
}

# Find any one of the note words definded above
RX_WORDS = re.compile("\\b({})\\b".format("|".join(WORD_CODES)))


def format_message(code: Code, word: str) -> Flake8Message:
    return Flake8Message("{} fixme found ({})".format(code, word))


def match_to_flake8_result(match: Match[str]) -> Flake8Result:
    word = match.group(1)
    code = WORD_CODES[word]
    result = Flake8Result((match.start(), format_message(code, word)))
    return result


def check(physical_line: str) -> Iterator[Flake8Result]:
    matches = RX_WORDS.finditer(physical_line)
    results = map(match_to_flake8_result, matches)
    yield from results


check.name = NAME  # type: ignore
check.version = VERSION  # type: ignore



import re
from typing import NewType

from .metadata import CODE_STEM

Code = NewType("Code", str)

RX_CODE_LEAF_VALID = re.compile(r"[1-9]\d+")


def code_from(code: str) -> Code:
    """Create an error code for this plugin from the given number"""
    # Check the numerical part of the error code is valid
    assert RX_CODE_LEAF_VALID.match(code)

    return Code("".join((CODE_STEM, code)))



# coding: utf-8

from __future__ import with_statement

from setuptools import setup

from flake8_fixme.metadata import CODE_STEM, NAME, VERSION


def get_readme() -> str:
    with open("README.md") as readme_handle:
        return readme_handle.read()


setup(
    name=NAME,
    version=VERSION,
    description="FIXME and TODO checker. Plugin for flake8.",  # noqa: T
    long_description=get_readme(),
    long_description_content_type="text/markdown",
    keywords="flake8 plugin fixme todo xxx hack",
    author="Tom Milligan",
    author_email="tommilligan@users.noreply.github.com",
    url="https://github.com/tommilligan/flake8-fixme",
    license="Apache-2.0",
    packages=["flake8_fixme"],
    install_requires=[],
    zip_safe=False,
    entry_points={"flake8.extension": ["{} = flake8_fixme:check".format(CODE_STEM)]},
    classifiers=[
        "Framework :: Flake8",
        "Intended Audience :: Developers",
        "Development Status :: 5 - Production/Stable",
        "Operating System :: OS Independent",
        "License :: OSI Approved :: Apache Software License",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Topic :: Software Development :: Quality Assurance",
    ],
)
